#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         ControlPanel.py
# Purpose:      This panel is used for control
#
#               Most of this code is from the wxPython demo application.
#
# Author:       Michael Broxton, Josh Lifton, and Daniel Alejandro Benitez
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#----------------------------------------------------------------------------

import wx
from wxPython.wx import *
from Globals import *
from string import *

ID_CP_SETCOLOR = wxNewId()
ID_CP_SETLEDSTATE = wxNewId()
ID_CP_SILENCE = wxNewId()
ID_CP_REBOOT = wxNewId()
ID_CP_SEND = wxNewId()
ID_CP_LEDSTATECHOICE = wxNewId()

class ControlPanel(wx.Panel):
    def __init__(self, parent, controller, menubar):
        wx.Panel.__init__(self, parent, -1)

        self.controller = controller
        self.menubar = menubar
        #Entry field for destination address
        textAddress = wx.StaticText(self, -1, "Destination Address: ")
        inputAddress = wx.TextCtrl(self, -1, "", size = (100,-1))
        self.iAddr = inputAddress

        #Button to send RGB value. NOTE: Destination address must be entered
        setColorbutton = wx.Button(self, ID_CP_SETCOLOR, "Set Color", (20, 20))
        setColorbutton.SetDefault()
        setColorbutton.SetSize(setColorbutton.GetBestSize())

        #Entry fields for RGB values
        textR = wx.StaticText(self, -1, "R:", size= (10,-1))
        inputR = wx.TextCtrl(self, -1, "", size = (25,-1))
        inputR.SetMaxLength(2)
        textG = wx.StaticText(self, -1, "G:", size= (10,-1))
        inputG = wx.TextCtrl(self, -1, "", size = (25,-1))
        inputG.SetMaxLength(2)
        textB = wx.StaticText(self, -1, "B:", size= (10,-1))
        inputB = wx.TextCtrl(self, -1, "", size = (25,-1))
        inputB.SetMaxLength(2)
        self.colors=[inputR,inputG,inputB]

        setLedStatebutton = wx.Button(self, ID_CP_SETLEDSTATE, "Set Led State", (20, 20))
        setLedStatebutton.SetDefault()
        setLedStatebutton.SetSize(setLedStatebutton.GetBestSize())
        
        LedStateList = ['', 'WHITE','RED','LIME','BLUE','AQUA','FUCHSIA',
                            'YELLOW','MAROON','GREEN', 'NAVY', 'TEAL', 'PURPLE',
                            'OLIVE', 'GRAY', 'SILVER', 'OFF',
                        '', 'REDLINEAR', 'REDPARA', 'GREENLINEAR', 'GREENPARA',
                            'BLUELINEAR','BLUEPARA', 'PULSE4', 'TOFCHILLIN',
                            'TOFEVERYONE', 'TOFANCHOR1', 'TOFANCHOR2',
                            'TOFANCHOR3', 'TOFBOUND1', 'TOFBOUND2','TEST']

        LedChoice = wx.Choice(self,ID_CP_LEDSTATECHOICE, choices = LedStateList)
        self.lc = LedChoice

        silencebutton = wx.Button(self, ID_CP_SILENCE, "Silence", (20, 20))
        silencebutton.SetDefault()
        silencebutton.SetSize(silencebutton.GetBestSize())

        rebootbutton = wx.Button(self, ID_CP_REBOOT, "Reboot", (20, 20))
        rebootbutton.SetDefault()
        rebootbutton.SetSize(rebootbutton.GetBestSize())

        messagetext = wx.StaticText(self, -1, "Generic Message: ")
        inputMessage = wx.TextCtrl(self, -1, "", size = (100,-1))
        self.iMsg = inputMessage
        
        sendbutton = wx.Button(self, ID_CP_SEND, "SEND", (20, 20))
        sendbutton.SetSize(sendbutton.GetBestSize())


        space = 20
        asizer = wx.BoxSizer(wx.HORIZONTAL)
        asizer.Add(textAddress, 0, wx.ALIGN_CENTER)
        asizer.Add((space, -1), 0)
        asizer.Add(inputAddress,0, wx.ALIGN_CENTER)

        space = 45
        bsizer = wx.BoxSizer(wx.HORIZONTAL)
        bsizer.Add(setColorbutton,0,wx.ALIGN_CENTER)
        bsizer.Add((space, -1), 0)
        space,lspace = 20,10
        bsizer.Add(textR, 0, wx.ALIGN_CENTER)
        bsizer.Add((lspace, -1), 0)
        bsizer.Add(inputR, 0, wx.ALIGN_CENTER)
        bsizer.Add((space, -1), 0)
        bsizer.Add(textG, 0, wx.ALIGN_CENTER)
        bsizer.Add((lspace, -1), 0)
        bsizer.Add(inputG, 0, wx.ALIGN_CENTER)
        bsizer.Add((space, -1), 0)
        bsizer.Add(textB, 0, wx.ALIGN_CENTER)
        bsizer.Add((lspace, -1), 0)
        bsizer.Add(inputB, 0, wx.ALIGN_CENTER)

        space = 50
        csizer = wx.BoxSizer(wx.HORIZONTAL)
        csizer.Add(setLedStatebutton, 0, wx.ALIGN_CENTER)
        csizer.Add((space, -1), 0)
        csizer.Add(LedChoice)

        space = 20
        dsizer = wx.BoxSizer(wx.HORIZONTAL)
        dsizer.Add(messagetext, 0, wx.ALIGN_CENTER)
        dsizer.Add((space, -1), 0)
        dsizer.Add(inputMessage, 0, wx.ALIGN_CENTER)
        space = 150
        dsizer.Add((space, -1), 0)
        dsizer.Add(sendbutton, 0, wx.ALIGN_CENTER)
        
        space = 10
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(asizer,0,wx.ALIGN_LEFT)
        sizer.Add((-1, space), 0)
        sizer.Add(bsizer,0,wx.ALIGN_LEFT)
        sizer.Add((-1, space), 0)
        sizer.Add(csizer,0,wx.ALIGN_LEFT)
        sizer.Add((-1, space), 0)
        sizer.Add(silencebutton,0,wx.ALIGN_LEFT)
        sizer.Add((-1, space), 0)
        sizer.Add(rebootbutton,0,wx.ALIGN_LEFT)
        sizer.Add((-1, space), 0)
        sizer.Add(dsizer,0,wx.ALIGN_LEFT)
        self.SetSizer(sizer)
        self.SetAutoLayout(True)

        #Bind Button Event
        self.Bind(wx.EVT_BUTTON, self.OnButton)


    #
    # This routine is called when this notebook panel becomes active.
    #
    def OnActivate(self):
        pass

    #
    # This routine is called when this notebook panel becomes inactive.
    #
    def OnDeactivate(self):
        pass

    #
    # This routine is used to determine what string should be used to label
    # this notebook tab.
    #
    def Name(self):
        return "Control Panel"



    def OnButton(self, event):
        
        #isaddressOK makes sure that the input of Destination Address is a valid one
        def isaddrOK():
            return isbyteOK(self.iAddr.GetValue())
            
        #isbyteOK makes sure that the input is either one or two chars long and that each char is a proper hexdigit        
        def isbyteOK(byte):
            if len(byte)==2:
                return ((byte[0] in hexdigits) and (byte[1] in hexdigits))
            else:
                return False

        #ismsgOK makes sure that the input has hexdigits and has an even number of bytes
        #it returns either the actual message (s) or False if message is OK or not, respectively
        def ismsgOK():
            msg = self.iMsg.GetValue()
            s = ""
            chars_read = 0     #This is a state variable to determine if msg is properly written
            
            if msg:
                for i in range(len(msg)):
                    if ((chars_read!=1) and (msg[i] == ' ' or msg[i] == ',')):
                        chars_read=0
                    elif ((chars_read!=2) and (msg[i] in hexdigits)):
                        chars_read += 1
                        s = s + msg[i]
                    else:
                        return False
            else: #msg is ''
                return False
                
            if (len(s)%2 == 0):
                return s
            else:
                return False
                
        if event.GetId() == ID_CP_SETCOLOR:
            if isaddrOK():
                for color in self.colors:
                    if isbyteOK(color.GetValue()):
                        isOK = True
                    else:
                        isOK = False
                        break
                if isOK:
                    hexcolor = ['','','']
                    for i in range(len(self.colors)):
                        hexcolor[i] = self.colors[i].GetValue()

                    self.controller.spotlightDataLink.sendBasicPacket(toAddress = self.iAddr.GetValue(),
                                                                   content=[SET_COLOR_MESSAGE,hexcolor[0],hexcolor[1],hexcolor[2]])
                    print "Color is set: " + join(hexcolor,sep="")
                else:
                    print >> self.controller.pd_stderr, "Inappropriate RGB values"
            else:
                print >> self.controller.pd_stderr, "Inappropriate Address" 
                
        elif event.GetId() == ID_CP_SETLEDSTATE:
            if isaddrOK():
                cur = self.lc.GetSelection()
                string = self.lc.GetString(cur)
                if string != '':
                    if cur < 17: #Take into account the initial space
                        cur -=1
                    else: #Take into account the space that separate Solid Led States from Active
                        cur -=2
                    cur = str('%02X' % cur)
                    self.controller.spotlightDataLink.sendBasicPacket(toAddress = self.iAddr.GetValue(),
                                                                   content=[SET_LED_STATE_MESSAGE, cur])
                    print "LED State is set:  "+string
                else:
                    print >> self.controller.pd_stderr, "Inappropriate LED State value"
            else:
                print >> self.controller.pd_stderr, "Inappropriate Address"
            
                
        elif event.GetId() == ID_CP_SILENCE:
##            if isaddrOK():
##                self.controller.dongleDataLink.sendBasicPacket(toAddress = self.iAddr.GetValue(),
##                                                               content=[BEGIN_DATA_HARVEST])
##            elif self.iAddr.GetValue()=="":
##                self.controller.dongleDataLink.sendSilencePacket()
##            else:
##                print >> self.controller.pd_stderr, "Inappropriate Address"
##                
            if isaddrOK():
                print self.iAddr.GetValue()
        elif event.GetId() == ID_CP_REBOOT:
            if isaddrOK():
                self.controller.dongleDataLink.sendBasicPacket(toAddress = self.iAddr.GetValue(),
                                                               content=[REBOOT_MESSAGE])
            elif self.iAddr.GetValue()=="":
                self.controller.dongleDataLink.sendBasicPacket(content=[REBOOT_MESSAGE])
            else:
                print >> self.controller.pd_stderr, "Inappropriate Address"
                
        elif event.GetId() == ID_CP_SEND:
            if isaddrOK():
                msg = ismsgOK()
                if msg:
                    self.controller.dongleDataLink.sendBasicPacket(toAddress = self.iAddr.GetValue(),
                                                                   content=[SENT_MESSAGE, msg])
                    print "Sent Message:  "+str(msg)
##                    self.iMsg.SetValue("")
##                    self.iAddr.SetValue("")
                else:
                    print >> self.controller.pd_stderr, "Inappropriate message to send"
            else:
                print >> self.controller.pd_stderr, "Inappropriate Address"

                


def runPanel (parent, controller, menubar):
    return ControlPanel(parent, controller, menubar)



