#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#----------------------------------------------------------------------------
# Name:         MonitorPanel.py
# Purpose:      This panel is used for monitoring
#
#               Most of this code is from the wxPython demo application.
#
# Author:       Michael Broxton, Josh Lifton, and Daniel Alejandro Benitez
#
# Created:      A long time ago, in a galaxy far, far away...
# Copyright:    (c) 2004 by MIT Media Laboratory
#----------------------------------------------------------------------------

import wx
from wxPython.wx import *
import wx.lib.mixins.listctrl  as  listmix
from ListPanel import PushpinListCtrl
from ListPanel import *
from Globals import *

ID_MP_LIST = wxNewId()
ID_MP_START = wxNewId()
ID_MP_STOP = wxNewId()
ID_MP_CLEAR = wxNewId()

class MonitorPanel(wx.Panel, listmix.ColumnSorterMixin):
    def __init__(self, parent, controller, menubar):
        wx.Panel.__init__(self, parent, -1, style=wx.WANTS_CHARS)

        self.controller = controller
        self.menubar = menubar
        self.list = PushpinListCtrl(self, ID_MP_LIST,
                                    size=(500,200),
                                    style=wx.LC_REPORT 
                                    | wx.SUNKEN_BORDER
                                    | wx.LC_EDIT_LABELS,
                                    # | wxLC_NO_HEADER
                                    # | wx.LC_VRULES | wx.LC_HRULES
                                    )

        self.list.ClearAll()
        
        #ADD/REMOVE Column labels here
        self.LegendText = [ 'To', 'From', 'Packet Type','Hop count', 'Content']
 
        # Add the columns from the data map legend.  The first column
        # will be the dictionary key, which isn't included in the legend.
        for z in range(len(self.LegendText)):
            self.list.InsertColumn( z, self.LegendText[z])


        #The following buttons: START, STOP, and CLEAR are placed below the list
        startbutton = wx.Button(self, ID_MP_START, "Start", (20, 20))
        startbutton.SetDefault()
        startbutton.SetSize(startbutton.GetBestSize())
        self.startb = startbutton
        
        stopbutton = wx.Button(self, ID_MP_STOP, "Stop", (20, 20))
        stopbutton.SetDefault()
        stopbutton.SetSize(stopbutton.GetBestSize())
        self.stopb = stopbutton
        
        clearbutton = wx.Button(self, ID_MP_CLEAR, "Clear", (20, 20))
        clearbutton.SetDefault()
        clearbutton.SetSize(clearbutton.GetBestSize())
        self.cb = clearbutton

        listmix.ColumnSorterMixin.__init__(self, 20)
        
        self.SetupLayout()

      
        
        #Bind Button Event
        self.Bind(wx.EVT_BUTTON, self.OnButton)


    #
    # This routine is called when this notebook panel becomes active.
    #
    def OnActivate(self):
        pass

    #
    # This routine is called when this notebook panel becomes inactive.
    #
    def OnDeactivate(self):
        pass

    #
    # This routine is used to determine what string should be used to label
    # this notebook tab.
    #
    def Name(self):
        return "Packet Monitor"


    def SetupLayout(self):
        #The buttons are in a horizontal sizer, and this sizer is below the list
        #in a vertical sizer
        asizer = wx.BoxSizer(wx.HORIZONTAL)
        asizer.Add(self.startb, 0, wx.ALIGN_CENTER)
        asizer.Add((40, -1), 0)
        asizer.Add(self.stopb,0, wx.ALIGN_CENTER)
        asizer.Add((140, -1), 0)
        asizer.Add(self.cb,0, wx.ALIGN_CENTER)
 
        space = 30
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.list,0, wx.EXPAND)
        sizer.Add((-1, space), 0)
        sizer.Add(asizer,0,wx.ALIGN_CENTER)
        self.SetSizer(sizer)
        self.SetAutoLayout(True)

        
    def OnButton(self, event):
        if event.GetId() == ID_MP_START:
            pass
        elif event.GetId() == ID_MP_STOP:
            pass
        elif event.GetId() == ID_MP_CLEAR:
            pass

    def colorListForReadability(self):
        # Color alternating rows with a light grey to enhance readability
        COLORLIST = [wx.Colour(230,230,230),wx.Colour(255, 255, 255)]
        for x in range( self.list.GetItemCount() ):
            item = self.list.GetItem(x)
            item.SetBackgroundColour(COLORLIST[x%2])
            self.list.SetItem(item)
           
##
##    def PopulateList(self):
##
##        # Add the actual data items themselves. 
##        items = self.itemDataMap.items()
##        for x in range(0,len(items)): 
##            key, data = items[x]
##            self.list.InsertStringItem(x, str("%02X" % key)) # Insert the key as a hex string
##            for y in range( len(data) ):
##                # MEME - Handle different data types appropriately!!!  This will
##                # probably be dself.Bind(wx.EVT_SIZE, self.OnSize)one in the NodeDirectory Class!!!
##                if ( ( y < len ( self.extraListLegendText ) ) and ( self.extraListLegendText[y] == ' ( meters ) ' ) ) :
##                    self.list.SetStringItem( x, y + 1, data[y] + ' ( ' + str( self.hexToSignedInt( data[y] ) / CLOCK_TICKS_PER_METER ) + ' ) ' )
##                else :
##                    self.list.SetStringItem( x, y + 1, data[y] )
##            self.list.SetItemData(x, key)
##
##        for w in range ( numberOfColumns ) :
##            if ( w < len( self.widthOfListColumns ) ) :
##                 self.list.SetColumnWidth( w, self.widthOfListColumns[w] )
##            else :
##                 self.list.SetColumnWidth( w, 120)
##
##        # Color the list for readability
##        self.colorListForReadability()

    # Used by the ColumnSorterMixin, see wxPython/lib/mixins/listctrl.py
    def GetListCtrl(self):
        return self.list
                      
def runPanel (parent, controller, menubar):
    return MonitorPanel(parent, controller, menubar)



