#
# Josh Lifton and Michael Broxton
# MIT Media Lab
# Copyright (c) 2002-2004. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

#
# Purpose:
#   - These classes form the basis of a simple command line text menu
#     interface.
#
# Assumptions:
#   - Menus must be built up by calling addCommand.
#   - The TextMenu's run command is the outer most control structure
#     of the program using the TextMenu, except possibly for other
#     threads.
#
# History:
#   - 16 Mar 2004 by Josh Lifton.  Added beforeEveryCommand and
#     afterEveryCommand attributes.
#   - 09 Oct 2003 by Josh Lifton.  Original TextMenu and MenuCommand
#     classes added.
#

class TextMenu :
    def __init__(self, title = 'Menu', prompt = '--> ', commands = [], beforeEveryCommand=None, afterEveryCommand=None) :
        self.title = title
        self.prompt = prompt
        self.commands = {}
        for item in commands :
            self.addCommand(item)
        self.beforeEveryCommand = beforeEveryCommand
        self.afterEveryCommand = afterEveryCommand

    def addCommand(self, mc) :
        if isinstance(mc, MenuCommand) :
            self.commands[mc.id] = mc

    def displayMenu(self) :
        maxCommandLength = max(map(len, self.commands.keys()))
        print(self.title)
        for id in self.commands.keys() :
            print(' '*(3 + maxCommandLength - len(id)) +
                  '\'' + id + '\' : ' + self.commands[id].desc)

    def run(self) :
        self.displayMenu()
        while 1 :
            userInput = raw_input(self.prompt)
            if self.commands.has_key(userInput) :
                if self.beforeEveryCommand != None :
                    self.beforeEveryCommand()
                self.commands[userInput].func()
                if self.afterEveryCommand != None :
                    self.afterEveryCommand()
            else :
                self.displayMenu()
                print('Please choose from the selection above.')

        
class MenuCommand :
    def __init__(self, id, desc, func) :
        self.id = id
        self.desc = desc
        self.func = func
